#include "value.h"

#include <string>

namespace domain {

Value::ValueType Value::getType() {
    return type;
}
bool Value::isType(Value::ValueType type) {
    return this->type == type;
}

long long Value::asInt() {
    if(isType(Value::INT)) {
        return intValue;
    }
    throw std::exception();
}
std::string Value::asString() {
    if(isType(Value::STRING)) {
        return stringValue;
    }
    throw std::exception();
}
bool Value::asBool() {
    if(isType(Value::BOOL)) {
        return boolValue;
    }
    throw std::exception();
}
AddressSpace Value::asAddressSpace() {
    if(isType(Value::ADDRESS_SPACE)) {
        return *addressSpace;
    }
    throw std::exception();
}
std::string Value::asReference() {
    if(isType(Value::WIRE_REFERENCE) || isType(Value::ATTRIBUTE_REFERENCE)) {
        return reference;
    }
    throw std::exception();
}

Value Value::fromInt(long long value) {
    Value val;
    val.type = Value::INT;
    val.intValue = value;
    return val;
}
Value Value::fromString(std::string value) {
    Value val;
    val.type = Value::STRING;
    val.stringValue = value;
    return val;
}
Value Value::fromBool(bool value) {
    Value val;
    val.type = Value::BOOL;
    val.boolValue = value;
    return val;
}
Value Value::fromAddressSpace(AddressSpace addressSpace) {
    Value val;
    val.type = Value::ADDRESS_SPACE;
    val.addressSpace = addressSpace;
    return val;
}
Value Value::fromReference(std::string value, Value::ValueType type) {
    Value val;
    val.type = type;
    val.reference = value;
    return val;
}
Value Value::ofType(Value::ValueType type) {
    Value val;
    val.type = type;
    return val;

}

} // namespace domain
